PROGRAM test_sort
!
!  This module tests sorting with parallel images..
!
!  Record of revisions:
!      Date       Programmer          Description of change
!      ====       ==========          =====================
!    05/06/16    S. J. Chapman        Original code
!
USE merge_module                       ! Merge 
USE timer_class                        ! Timer class
IMPLICIT NONE

! Parameters
INTEGER,PARAMETER :: N_SAMPLES = 100000

! Declare variables
REAL,DIMENSION(N_SAMPLES) :: a      ! Input values to sort
REAL,DIMENSION(N_SAMPLES/2) :: b[*] ! Coarray for parallel sorting
REAL,DIMENSION(N_SAMPLES/2) :: b1   ! Arrays for sequential sorting
REAL,DIMENSION(N_SAMPLES/2) :: b2   ! Arrays for sequential sorting
REAL :: elapsed_time                ! Elapsed time (s)
INTEGER :: i                        ! Loop index
INTEGER :: m                        ! Number of values to sort
REAL,DIMENSION(N_SAMPLES) :: out    ! Sorted output values 
TYPE(timer) :: t                    ! Timer object

!*************************************************
!*************************************************
! Now sort the data using two images
!*************************************************
!*************************************************

!*************************************************
! Create the input array using Image 1
!*************************************************
IF ( this_image() == 1 ) THEN

   ! Allocate the data to sort
   CALL random_number(a)
   
   ! Start the timer
   CALL t%start_timer()
   
   ! Copy the data into the working arrays
   ! for each image.
   b[1] = a(1:N_SAMPLES/2)
   b[2] = a(N_SAMPLES/2+1:N_SAMPLES)
   
END IF

!*************************************************
! Synchronise all images during the creation of 
! the input data
!*************************************************
SYNC ALL

!*************************************************
! Now all images can run in parallel.
! Sort the data in each image
!*************************************************
CALL sort( b, N_SAMPLES/2 )

!*************************************************
! Wait until all images are finished
!*************************************************
SYNC ALL

!*************************************************
! Now merge the data back into a common output 
! array using image 1, and display the results.
!*************************************************
IF ( this_image() == 1 ) THEN

   ! Merge the data
   CALL merge ( b[1], N_SAMPLES/2, b[2], N_SAMPLES/2, out, N_SAMPLES )
   
   ! Stop the timer
   elapsed_time = t%elapsed_time()
   
   ! Display the elapsed time
   WRITE (*,'(A,F8.3,A)') &
      'Parallel sort elapsed time =', elapsed_time, ' s'
      
   ! Display first 5 samples
   WRITE (*,'(A)') 'First 5 samples:'
   DO i = 1, 5
      WRITE (*,'(F10.6)') out(i)
   END DO
   
   ! Display last 5 samples
   WRITE (*,'(A)') 'Last 5 samples:'
   DO i = N_SAMPLES-4, N_SAMPLES
      WRITE (*,'(F10.6)') out(i)
   END DO
   
ELSE

   ! Stop other images--they won't be used again
   STOP
   
END IF

!*************************************************
!*************************************************
! Now sort the data using a single image
!*************************************************
!*************************************************
IF ( this_image() == 1 ) THEN

   ! Start the timer
   CALL t%start_timer()
   
   ! Copy the data into the working arrays
   b1 = a(1:N_SAMPLES/2)
   b2 = a(N_SAMPLES/2+1:N_SAMPLES)

   ! Sort the data in the single image
   CALL sort( b1, N_SAMPLES/2 )
   CALL sort( b2, N_SAMPLES/2 )

   ! Merge the data
   CALL merge ( b1, N_SAMPLES/2, b2, N_SAMPLES/2, out, N_SAMPLES )
   
   ! Stop the timer
   elapsed_time = t%elapsed_time()
   
   ! Display the elapsed time
   WRITE (*,'(/A,F8.3,A)') &
      'Sequential sort elapsed time =', elapsed_time, ' s'
      
   ! Display first 5 samples
   WRITE (*,'(A)') 'First 5 samples:'
   DO i = 1, 5
      WRITE (*,'(F10.6)') out(i)
   END DO
   
   ! Display last 5 samples
   WRITE (*,'(A)') 'Last 5 samples:'
   DO i = N_SAMPLES-4, N_SAMPLES
      WRITE (*,'(F10.6)') out(i)
   END DO
   
END IF

END PROGRAM test_sort
